@@ -6,7 +6,7 @@ use 5.010;
 
 use parent 'Hypersonic::Event::Role';
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 sub name { 'epoll' }
 
@@ -66,7 +66,7 @@ sub gen_del {
     $builder->line("epoll_ctl($loop_var, EPOLL_CTL_DEL, $fd_var, NULL);");
 }
 
-# Generate: wait for events
+# Generate: wait for events (inside a loop - can use continue/break)
 sub gen_wait {
     my ($class, $builder, $loop_var, $events_var, $count_var, $timeout_var) = @_;
 
@@ -80,6 +80,14 @@ sub gen_wait {
       ->endif;
 }
 
+# Generate: wait for events - single call version (no loop control)
+sub gen_wait_once {
+    my ($class, $builder, $loop_var, $events_var, $count_var, $timeout_ms) = @_;
+
+    $builder->line("int $count_var = epoll_wait($loop_var, $events_var, MAX_EVENTS, $timeout_ms);")
+      ->line("if ($count_var < 0 && errno == EINTR) $count_var = 0;");
+}
+
 # Generate: extract fd from event
 sub gen_get_fd {
     my ($class, $builder, $events_var, $index_var, $fd_var) = @_;
@@ -87,6 +95,47 @@ sub gen_get_fd {
     $builder->line("int $fd_var = ${events_var}[$index_var].data.fd;");
 }
 
+# ============================================================
+# Async Slot Integration Methods (UA Async)
+# ============================================================
+
+# Generate: create epoll without adding any fds
+# Note: loop_var must be already declared
+sub gen_create_loop {
+    my ($class, $builder, $loop_var) = @_;
+    
+    $builder->line("$loop_var = epoll_create1(0);")
+      ->if("$loop_var < 0")
+        ->line('croak("epoll_create1() failed");')
+      ->endif
+      ->line('struct epoll_event ev;');
+}
+
+# Generate: add fd with slot as user data
+# Note: epoll_event.data is a union - we use data.u32 for slot
+sub gen_add_with_slot {
+    my ($class, $builder, $loop_var, $fd_var, $slot_var, $events) = @_;
+    
+    my $ev_flags = $events eq 'read' ? 'EPOLLIN | EPOLLET | EPOLLONESHOT' 
+                 : $events eq 'write' ? 'EPOLLOUT | EPOLLET | EPOLLONESHOT'
+                 : 'EPOLLIN | EPOLLET | EPOLLONESHOT';
+    
+    $builder->line("ev.events = $ev_flags;")
+      ->line("ev.data.u32 = (uint32_t)$slot_var;")
+      ->line("epoll_ctl($loop_var, EPOLL_CTL_ADD, $fd_var, &ev);");
+}
+
+# Generate: extract slot from event data
+sub gen_get_slot {
+    my ($class, $builder, $events_var, $index_var, $slot_var) = @_;
+    
+    $builder->line("int $slot_var = (int)${events_var}[$index_var].data.u32;");
+}
+
+# ============================================================
+# Future/Pool Integration Methods
+# ============================================================
+
 # Future/Pool integration - add pool notify fd to epoll
 sub gen_add_pool_notify {
     my ($class, $builder, $loop_var, $notify_fd_var) = @_;
