@@ -6,7 +6,7 @@ use 5.010;
 
 use parent 'Hypersonic::Event::Role';
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 sub name { 'kqueue' }
 
@@ -65,7 +65,7 @@ sub gen_del {
       ->line("kevent($loop_var, &ev, 1, NULL, 0, NULL);");
 }
 
-# Generate: wait for events with timeout
+# Generate: wait for events with timeout (inside a loop - can use continue/break)
 sub gen_wait {
     my ($class, $builder, $loop_var, $events_var, $count_var, $timeout_var) = @_;
 
@@ -83,6 +83,18 @@ sub gen_wait {
       ->endif;
 }
 
+# Generate: wait for events - single call version (no loop control)
+sub gen_wait_once {
+    my ($class, $builder, $loop_var, $events_var, $count_var, $timeout_ms) = @_;
+
+    $builder->line('struct timespec ts;')
+      ->line("ts.tv_sec = $timeout_ms / 1000;")
+      ->line("ts.tv_nsec = ($timeout_ms % 1000) * 1000000;")
+      ->blank
+      ->line("int $count_var = kevent($loop_var, NULL, 0, $events_var, MAX_EVENTS, &ts);")
+      ->line("if ($count_var < 0 && errno == EINTR) $count_var = 0;");
+}
+
 # Generate: extract fd from event
 sub gen_get_fd {
     my ($class, $builder, $events_var, $index_var, $fd_var) = @_;
@@ -90,6 +102,48 @@ sub gen_get_fd {
     $builder->line("int $fd_var = (int)${events_var}[$index_var].ident;");
 }
 
+# ============================================================
+# Async Slot Integration Methods (UA Async)
+# ============================================================
+
+# Generate: create kqueue without adding any fds
+# Note: loop_var must be already declared
+sub gen_create_loop {
+    my ($class, $builder, $loop_var) = @_;
+    
+    $builder->line("$loop_var = kqueue();")
+      ->if("$loop_var < 0")
+        ->line('croak("kqueue() failed");')
+      ->endif
+      ->line('struct kevent ev;');
+}
+
+# Generate: add fd with slot as user data
+sub gen_add_with_slot {
+    my ($class, $builder, $loop_var, $fd_var, $slot_var, $events) = @_;
+    
+    my $filter = $events eq 'read' ? 'EVFILT_READ' 
+               : $events eq 'write' ? 'EVFILT_WRITE'
+               : 'EVFILT_READ';  # default to read
+    
+    $builder->line('{')
+      ->line('    struct kevent _ev;')
+      ->line("    EV_SET(&_ev, $fd_var, $filter, EV_ADD | EV_ENABLE | EV_ONESHOT, 0, 0, (void *)(intptr_t)$slot_var);")
+      ->line("    kevent($loop_var, &_ev, 1, NULL, 0, NULL);")
+      ->line('}');
+}
+
+# Generate: extract slot from event udata
+sub gen_get_slot {
+    my ($class, $builder, $events_var, $index_var, $slot_var) = @_;
+    
+    $builder->line("int $slot_var = (int)(intptr_t)${events_var}[$index_var].udata;");
+}
+
+# ============================================================
+# Future/Pool Integration Methods
+# ============================================================
+
 # Future/Pool integration - add pool notify fd to kqueue
 sub gen_add_pool_notify {
     my ($class, $builder, $loop_var, $notify_fd_var) = @_;
