@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 # Hypersonic::Event::Role - Base class for event backends
 #
@@ -88,6 +88,32 @@ sub gen_cleanup {
     # Default: no-op
 }
 
+# ============================================================
+# Async Slot Integration Methods (UA Async)
+# ============================================================
+
+# Generate: add fd with slot as user data
+# slot_var is an integer slot ID that will be stored with the fd
+# events: 'read', 'write', or 'both'
+sub gen_add_with_slot {
+    my ($class, $builder, $loop_var, $fd_var, $slot_var, $events) = @_;
+    # Default implementation - subclasses should override
+    die "$class must implement gen_add_with_slot(\$builder, \$loop_var, \$fd_var, \$slot_var, \$events)";
+}
+
+# Generate: extract slot from event (returns the user data)
+sub gen_get_slot {
+    my ($class, $builder, $events_var, $index_var, $slot_var) = @_;
+    die "$class must implement gen_get_slot(\$builder, \$events_var, \$index_var, \$slot_var)";
+}
+
+# Generate: create event loop (without listen socket)
+sub gen_create_loop {
+    my ($class, $builder, $loop_var) = @_;
+    # Default - call gen_create with dummy fd
+    die "$class must implement gen_create_loop(\$builder, \$loop_var)";
+}
+
 # ============================================================
 # Future/Pool Integration Methods (Single Pool - Legacy)
 # ============================================================
