@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 # High-performance JIT-compiled Future for async operations
 # Serves as the foundation for all async in Hypersonic:
@@ -486,19 +486,16 @@ sub _gen_helpers {
             ->line('        } else {')
             ->line('            parent->subfutures_pending--;')
             ->line('            if (parent->subfutures_pending == 0) {')
-            ->line('                /* All done - collect results */')
-            ->line('                AV *results = newAV();')
-            ->line('                for (int i = 0; i < parent->subfuture_count; i++) {')
+            ->line('                /* All done - collect results as flat list */')
+            ->line('                int count = parent->subfuture_count;')
+            ->line('                parent->result_values = (SV**)malloc(count * sizeof(SV*));')
+            ->line('                parent->result_count = 0;')
+            ->line('                for (int i = 0; i < count; i++) {')
             ->line('                    FutureContext *sub = &future_registry[parent->subfuture_slots[i]];')
-            ->line('                    AV *sub_result = newAV();')
-            ->line('                    for (int j = 0; j < sub->result_count; j++) {')
-            ->line('                        av_push(sub_result, SvREFCNT_inc(sub->result_values[j]));')
+            ->line('                    if (sub->result_count > 0) {')
+            ->line('                        parent->result_values[parent->result_count++] = SvREFCNT_inc(sub->result_values[0]);')
             ->line('                    }')
-            ->line('                    av_push(results, newRV_noinc((SV*)sub_result));')
             ->line('                }')
-            ->line('                parent->result_values = (SV**)malloc(sizeof(SV*));')
-            ->line('                parent->result_values[0] = newRV_noinc((SV*)results);')
-            ->line('                parent->result_count = 1;')
             ->line('                parent->state = FUTURE_STATE_DONE;')
             ->line('                future_invoke_callbacks(parent_slot, FUTURE_CB_DONE);')
             ->line('            }')
@@ -1015,19 +1012,15 @@ sub _gen_xs_functions {
             ->line('    }')
             ->line('}')
             ->line('if (ctx->state == FUTURE_STATE_PENDING && all_done) {')
-            ->line('    /* All already done - collect results */')
-            ->line('    AV *results = newAV();')
+            ->line('    /* All already done - collect results as flat list */')
+            ->line('    ctx->result_values = (SV**)malloc(count * sizeof(SV*));')
+            ->line('    ctx->result_count = 0;')
             ->line('    for (int i = 0; i < count; i++) {')
             ->line('        FutureContext *sub = &future_registry[ctx->subfuture_slots[i]];')
-            ->line('        AV *sub_result = newAV();')
-            ->line('        for (int j = 0; j < sub->result_count; j++) {')
-            ->line('            av_push(sub_result, SvREFCNT_inc(sub->result_values[j]));')
+            ->line('        if (sub->result_count > 0) {')
+            ->line('            ctx->result_values[ctx->result_count++] = SvREFCNT_inc(sub->result_values[0]);')
             ->line('        }')
-            ->line('        av_push(results, newRV_noinc((SV*)sub_result));')
             ->line('    }')
-            ->line('    ctx->result_values = (SV**)malloc(sizeof(SV*));')
-            ->line('    ctx->result_values[0] = newRV_noinc((SV*)results);')
-            ->line('    ctx->result_count = 1;')
             ->line('    ctx->state = FUTURE_STATE_DONE;')
             ->line('}')
             ->line('SV *slot_sv = newSViv(new_slot);')
@@ -1135,18 +1128,15 @@ sub _gen_xs_functions {
             ->line('    }')
             ->line('}')
             ->line('if (ctx->state == FUTURE_STATE_PENDING && all_done) {')
-            ->line('    AV *results = newAV();')
+            ->line('    /* All already done - collect results as flat list */')
+            ->line('    ctx->result_values = (SV**)malloc(count * sizeof(SV*));')
+            ->line('    ctx->result_count = 0;')
             ->line('    for (int i = 0; i < count; i++) {')
             ->line('        FutureContext *sub = &future_registry[ctx->subfuture_slots[i]];')
-            ->line('        AV *sub_result = newAV();')
-            ->line('        for (int j = 0; j < sub->result_count; j++) {')
-            ->line('            av_push(sub_result, SvREFCNT_inc(sub->result_values[j]));')
+            ->line('        if (sub->result_count > 0) {')
+            ->line('            ctx->result_values[ctx->result_count++] = SvREFCNT_inc(sub->result_values[0]);')
             ->line('        }')
-            ->line('        av_push(results, newRV_noinc((SV*)sub_result));')
             ->line('    }')
-            ->line('    ctx->result_values = (SV**)malloc(sizeof(SV*));')
-            ->line('    ctx->result_values[0] = newRV_noinc((SV*)results);')
-            ->line('    ctx->result_count = 1;')
             ->line('    ctx->state = FUTURE_STATE_DONE;')
             ->line('}')
             ->line('SV *slot_sv = newSViv(new_slot);')
